Type: Package
Package: parameters
Title: Processing of Model Parameters
Version: 0.29.1
Authors@R: 
    c(person(given = "Daniel",
             family = "Lüdecke",
             role = c("aut", "cre"),
             email = "officialeasystats@gmail.com",
             comment = c(ORCID = "0000-0002-8895-3206")),
      person(given = "Dominique",
             family = "Makowski",
             role = "aut",
             email = "dom.makowski@gmail.com",
             comment = c(ORCID = "0000-0001-5375-9967")),
      person(given = "Mattan S.",
             family = "Ben-Shachar",
             role = "aut",
             email = "matanshm@post.bgu.ac.il",
             comment = c(ORCID = "0000-0002-4287-4801")),
      person(given = "Indrajeet",
             family = "Patil",
             role = "aut",
             email = "patilindrajeet.science@gmail.com",
             comment = c(ORCID = "0000-0003-1995-6531")),
      person(given = "Søren",
             family = "Højsgaard",
             role = "aut",
             email = "sorenh@math.aau.dk"),
      person(given = "Brenton M.",
             family = "Wiernik",
             role = "aut",
             email = "brenton@wiernik.org",
             comment = c(ORCID = "0000-0001-9560-6336")),
      person(given = "Zen J.",
             family = "Lau",
             role = "ctb",
             email = "zenjuen.lau@ntu.edu.sg"),
      person(given = "Vincent",
             family = "Arel-Bundock",
             role = "ctb",
             email = "vincent.arel-bundock@umontreal.ca",
             comment = c(ORCID = "0000-0003-2042-7063")),
      person(given = "Jeffrey",
             family = "Girard",
             role = "ctb",
             email = "me@jmgirard.com",
             comment = c(ORCID = "0000-0002-7359-3746")),
      person(given = "Christina",
             family = "Maimone",
             role = "rev",
             email = "christina.maimone@northwestern.edu"),
      person(given = "Niels",
             family = "Ohlsen",
             role = "rev"),
      person(given = "Douglas Ezra",
             family = "Morrison",
             role = "ctb",
             email = "dmorrison01@ucla.edu",
             comment = c(ORCID = "0000-0002-7195-830X")),
      person(given = "Joseph",
             family = "Luchman",
             role = "ctb",
             email = "jluchman@gmail.com",
             comment = c(ORCID = "0000-0002-8886-9717")))
Maintainer: Daniel Lüdecke <officialeasystats@gmail.com>
Description: Utilities for processing the parameters of various
    statistical models. Beyond computing p values, CIs, and other indices
    for a wide variety of models (see list of supported models using the
    function 'insight::supported_models()'), this package implements
    features like bootstrapping or simulating of parameters and models,
    feature reduction (feature extraction and variable selection) as well
    as functions to describe data and variable characteristics (e.g.
    skewness, kurtosis, smoothness or distribution).
License: GPL-3
URL: https://easystats.github.io/parameters/
BugReports: https://github.com/easystats/parameters/issues
Depends: R (>= 3.6)
Imports: bayestestR (>= 0.17.0), datawizard (>= 1.3.1), insight (>=
        1.5.0), graphics, methods, stats, utils
Suggests: AER, afex, aod, BayesFactor (>= 0.9.12-4.7), BayesFM, bbmle,
        BSDA, betareg, BH, biglm, blme, boot, brglm2, brms, broom,
        broom.mixed, cAIC4, car, carData, cgam, ClassDiscovery,
        clubSandwich, cluster, cobalt, coda, correlation (>= 0.8.8),
        coxme, cplm, curl, dbscan, did, discovr, distributional, domir
        (>= 0.2.0), drc, DRR, effectsize (>= 1.0.1), EGAnet, emmeans
        (>= 1.7.0), epiR, estimatr, factoextra, FactoMineR, faraway,
        fastICA, fixest, fpc, gam, gamlss, gee, geepack, ggplot2,
        GLMMadaptive, glmmTMB (>= 1.1.12), glmtoolbox, GPArotation, gt,
        haven, httr2, Hmisc, ivreg, knitr, lavaan, lavaan.mi, lcmm,
        lfe, lm.beta, lme4, lmerTest, lmtest, logistf, logitr,
        logspline, lqmm, M3C, marginaleffects (>= 0.29.0), modelbased
        (>= 0.9.0), MASS, Matrix, mclogit, mclust, MCMCglmm, mediation,
        merDeriv, metaBMA, metafor, mfx, mgcv, mice (>= 3.17.0), mmrm,
        multcomp, MuMIn, mvtnorm, NbClust, nFactors, nestedLogit, nlme,
        nnet, openxlsx, ordinal, panelr, pbkrtest, PCDimension,
        performance (>= 0.14.0), plm, PMCMRplus, poorman, posterior,
        PROreg (>= 1.3.0), pscl, psych, pvclust, quantreg,
        randomForest, RcppEigen, rmarkdown, rms, rstan, rstanarm,
        sampleSelection, sandwich, see (>= 0.8.1), serp, sparsepca,
        survey, survival, svylme, testthat (>= 3.2.1), tidyselect,
        tinytable (>= 0.13.0), TMB, truncreg, vdiffr, VGAM, WeightIt
        (>= 1.2.0), withr, WRS2
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
Config/testthat/edition: 3
Config/testthat/parallel: false
Config/Needs/website: easystats/easystatstemplate
Config/Needs/check: stan-dev/cmdstanr
Config/rcmdcheck/ignore-inconsequential-notes: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 11:01:56 UTC; DL
Author: Daniel Lüdecke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8895-3206>),
  Dominique Makowski [aut] (ORCID:
    <https://orcid.org/0000-0001-5375-9967>),
  Mattan S. Ben-Shachar [aut] (ORCID:
    <https://orcid.org/0000-0002-4287-4801>),
  Indrajeet Patil [aut] (ORCID: <https://orcid.org/0000-0003-1995-6531>),
  Søren Højsgaard [aut],
  Brenton M. Wiernik [aut] (ORCID:
    <https://orcid.org/0000-0001-9560-6336>),
  Zen J. Lau [ctb],
  Vincent Arel-Bundock [ctb] (ORCID:
    <https://orcid.org/0000-0003-2042-7063>),
  Jeffrey Girard [ctb] (ORCID: <https://orcid.org/0000-0002-7359-3746>),
  Christina Maimone [rev],
  Niels Ohlsen [rev],
  Douglas Ezra Morrison [ctb] (ORCID:
    <https://orcid.org/0000-0002-7195-830X>),
  Joseph Luchman [ctb] (ORCID: <https://orcid.org/0000-0002-8886-9717>)
Repository: CRAN
Date/Publication: 2026-05-24 12:20:02 UTC
Built: R 4.5.2; ; 2026-05-24 13:55:04 UTC; unix
