# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,VarCorr.coxme)
S3method(as.data.frame,VarCorr.lme)
S3method(as.double,n_clusters)
S3method(as.double,n_factors)
S3method(as.numeric,n_clusters)
S3method(as.numeric,n_factors)
S3method(bootstrap_model,default)
S3method(bootstrap_model,glmmTMB)
S3method(bootstrap_model,merMod)
S3method(bootstrap_model,nestedLogit)
S3method(bootstrap_parameters,bootstrap_model)
S3method(bootstrap_parameters,default)
S3method(ci,BBmm)
S3method(ci,BBreg)
S3method(ci,DirichletRegModel)
S3method(ci,HLfit)
S3method(ci,MixMod)
S3method(ci,Sarlm)
S3method(ci,averaging)
S3method(ci,bayesx)
S3method(ci,betamfx)
S3method(ci,betaor)
S3method(ci,betareg)
S3method(ci,bracl)
S3method(ci,brmultinom)
S3method(ci,btergm)
S3method(ci,clm2)
S3method(ci,clmm2)
S3method(ci,coeftest)
S3method(ci,complmrob)
S3method(ci,coxph)
S3method(ci,crq)
S3method(ci,default)
S3method(ci,deltaMethod)
S3method(ci,effectsize_table)
S3method(ci,estimate_contrasts)
S3method(ci,estimate_means)
S3method(ci,estimate_slopes)
S3method(ci,fixest_multi)
S3method(ci,flac)
S3method(ci,flic)
S3method(ci,gam)
S3method(ci,gamm)
S3method(ci,gamm4)
S3method(ci,geeglm)
S3method(ci,glht)
S3method(ci,glm)
S3method(ci,glmgee)
S3method(ci,glmm)
S3method(ci,glmmTMB)
S3method(ci,hglm)
S3method(ci,hurdle)
S3method(ci,ivFixed)
S3method(ci,ivprobit)
S3method(ci,lavaan)
S3method(ci,list)
S3method(ci,lm)
S3method(ci,lm_robust)
S3method(ci,lme)
S3method(ci,lmodel2)
S3method(ci,logistf)
S3method(ci,logitmfx)
S3method(ci,logitor)
S3method(ci,lqm)
S3method(ci,lqmm)
S3method(ci,margins)
S3method(ci,maxLik)
S3method(ci,mediate)
S3method(ci,merMod)
S3method(ci,merModList)
S3method(ci,meta_bma)
S3method(ci,meta_fixed)
S3method(ci,meta_random)
S3method(ci,metaplus)
S3method(ci,mhurdle)
S3method(ci,mipo)
S3method(ci,mira)
S3method(ci,mixor)
S3method(ci,mjoint)
S3method(ci,mle)
S3method(ci,mle2)
S3method(ci,mlm)
S3method(ci,mmrm)
S3method(ci,mmrm_fit)
S3method(ci,mmrm_tmb)
S3method(ci,model_fit)
S3method(ci,multinom)
S3method(ci,multinom_weightit)
S3method(ci,negbin)
S3method(ci,negbinirr)
S3method(ci,negbinmfx)
S3method(ci,nestedLogit)
S3method(ci,nlrq)
S3method(ci,ordinal_weightit)
S3method(ci,parameters_standardized)
S3method(ci,pgmm)
S3method(ci,phyloglm)
S3method(ci,phylolm)
S3method(ci,poissonirr)
S3method(ci,poissonmfx)
S3method(ci,polr)
S3method(ci,probitmfx)
S3method(ci,rma)
S3method(ci,rq)
S3method(ci,rqs)
S3method(ci,rqss)
S3method(ci,scam)
S3method(ci,selection)
S3method(ci,summary.lm)
S3method(ci,svyglm)
S3method(ci,svyolr)
S3method(ci,systemfit)
S3method(ci,varest)
S3method(ci,zerocount)
S3method(ci,zeroinfl)
S3method(ci_kenward,default)
S3method(ci_kenward,glmmTMB)
S3method(cluster_discrimination,cluster_analysis)
S3method(cluster_discrimination,default)
S3method(cluster_performance,dbscan)
S3method(cluster_performance,hclust)
S3method(cluster_performance,kmeans)
S3method(cluster_performance,parameters_clusters)
S3method(convert_efa_to_cfa,fa)
S3method(convert_efa_to_cfa,fa.ci)
S3method(convert_efa_to_cfa,parameters_efa)
S3method(convert_efa_to_cfa,parameters_pca)
S3method(display,compare_parameters)
S3method(display,equivalence_test_lm)
S3method(display,parameters_brms_meta)
S3method(display,parameters_efa)
S3method(display,parameters_efa_summary)
S3method(display,parameters_model)
S3method(display,parameters_omega)
S3method(display,parameters_omega_summary)
S3method(display,parameters_p_function)
S3method(display,parameters_pca)
S3method(display,parameters_pca_summary)
S3method(display,parameters_sem)
S3method(display,parameters_simulate)
S3method(display,parameters_standardized)
S3method(equivalence_test,MixMod)
S3method(equivalence_test,estimate_contrasts)
S3method(equivalence_test,estimate_means)
S3method(equivalence_test,estimate_slopes)
S3method(equivalence_test,feis)
S3method(equivalence_test,felm)
S3method(equivalence_test,gee)
S3method(equivalence_test,glm)
S3method(equivalence_test,glmmTMB)
S3method(equivalence_test,gls)
S3method(equivalence_test,hurdle)
S3method(equivalence_test,lm)
S3method(equivalence_test,lme)
S3method(equivalence_test,merMod)
S3method(equivalence_test,mixed)
S3method(equivalence_test,parameters_model)
S3method(equivalence_test,parameters_simulate_model)
S3method(equivalence_test,rma)
S3method(equivalence_test,wbm)
S3method(equivalence_test,zeroinfl)
S3method(factor_analysis,data.frame)
S3method(factor_analysis,matrix)
S3method(factor_scores,fa)
S3method(factor_scores,omega)
S3method(factor_scores,parameters_efa)
S3method(factor_scores,parameters_omega)
S3method(format,compare_parameters)
S3method(format,equivalence_test_lm)
S3method(format,p_calibrate)
S3method(format,parameters_brms_meta)
S3method(format,parameters_coef)
S3method(format,parameters_model)
S3method(format,parameters_p_function)
S3method(format,parameters_sem)
S3method(format,parameters_simulate)
S3method(format,parameters_standardized)
S3method(format_parameters,default)
S3method(format_parameters,emm_list)
S3method(format_parameters,glmm)
S3method(format_parameters,margins)
S3method(format_parameters,mediate)
S3method(format_parameters,merModList)
S3method(format_parameters,meta_bma)
S3method(format_parameters,meta_fixed)
S3method(format_parameters,meta_random)
S3method(format_parameters,mira)
S3method(format_parameters,mle2)
S3method(format_parameters,parameters_model)
S3method(format_parameters,rma)
S3method(model_parameters,AKP)
S3method(model_parameters,Anova.mlm)
S3method(model_parameters,BFBayesFactor)
S3method(model_parameters,BGGM)
S3method(model_parameters,DirichletRegModel)
S3method(model_parameters,FAMD)
S3method(model_parameters,Gam)
S3method(model_parameters,HLfit)
S3method(model_parameters,MCMCglmm)
S3method(model_parameters,Mclust)
S3method(model_parameters,MixMod)
S3method(model_parameters,PCA)
S3method(model_parameters,PMCMR)
S3method(model_parameters,SemiParBIV)
S3method(model_parameters,afex_aov)
S3method(model_parameters,anova)
S3method(model_parameters,anova.rms)
S3method(model_parameters,aov)
S3method(model_parameters,aovlist)
S3method(model_parameters,asym)
S3method(model_parameters,averaging)
S3method(model_parameters,bamlss)
S3method(model_parameters,bayesQR)
S3method(model_parameters,bcplm)
S3method(model_parameters,befa)
S3method(model_parameters,betamfx)
S3method(model_parameters,betaor)
S3method(model_parameters,betareg)
S3method(model_parameters,bfsl)
S3method(model_parameters,bifeAPEs)
S3method(model_parameters,blavaan)
S3method(model_parameters,blrm)
S3method(model_parameters,bootstrap_model)
S3method(model_parameters,bracl)
S3method(model_parameters,brmsfit)
S3method(model_parameters,brmultinom)
S3method(model_parameters,censReg)
S3method(model_parameters,cgam)
S3method(model_parameters,clm2)
S3method(model_parameters,clmm)
S3method(model_parameters,clmm2)
S3method(model_parameters,coeftest)
S3method(model_parameters,compare.loo)
S3method(model_parameters,comparisons)
S3method(model_parameters,coxme)
S3method(model_parameters,cpglmm)
S3method(model_parameters,data.frame)
S3method(model_parameters,dbscan)
S3method(model_parameters,default)
S3method(model_parameters,deltaMethod)
S3method(model_parameters,dep.effect)
S3method(model_parameters,draws)
S3method(model_parameters,emmGrid)
S3method(model_parameters,emm_list)
S3method(model_parameters,epi.2by2)
S3method(model_parameters,estimate_contrasts)
S3method(model_parameters,estimate_means)
S3method(model_parameters,estimate_slopes)
S3method(model_parameters,externVar)
S3method(model_parameters,externX)
S3method(model_parameters,fa)
S3method(model_parameters,fa.ci)
S3method(model_parameters,feglm)
S3method(model_parameters,fitdistr)
S3method(model_parameters,fixest)
S3method(model_parameters,fixest_multi)
S3method(model_parameters,flac)
S3method(model_parameters,flic)
S3method(model_parameters,gam)
S3method(model_parameters,gamlss)
S3method(model_parameters,gamm)
S3method(model_parameters,glht)
S3method(model_parameters,glimML)
S3method(model_parameters,glm)
S3method(model_parameters,glmm)
S3method(model_parameters,glmmTMB)
S3method(model_parameters,glmx)
S3method(model_parameters,hclust)
S3method(model_parameters,hdbscan)
S3method(model_parameters,hglm)
S3method(model_parameters,hkmeans)
S3method(model_parameters,htest)
S3method(model_parameters,hurdle)
S3method(model_parameters,hypotheses)
S3method(model_parameters,item_omega)
S3method(model_parameters,ivFixed)
S3method(model_parameters,ivprobit)
S3method(model_parameters,kmeans)
S3method(model_parameters,lavaan)
S3method(model_parameters,lavaan.mi)
S3method(model_parameters,lcmm)
S3method(model_parameters,list)
S3method(model_parameters,lm_robust)
S3method(model_parameters,lme)
S3method(model_parameters,lmodel2)
S3method(model_parameters,logistf)
S3method(model_parameters,logitmfx)
S3method(model_parameters,logitor)
S3method(model_parameters,lqm)
S3method(model_parameters,lqmm)
S3method(model_parameters,maov)
S3method(model_parameters,marginaleffects)
S3method(model_parameters,margins)
S3method(model_parameters,maxLik)
S3method(model_parameters,maxim)
S3method(model_parameters,mblogit)
S3method(model_parameters,mcmc)
S3method(model_parameters,mcmc.list)
S3method(model_parameters,mcp1)
S3method(model_parameters,mcp2)
S3method(model_parameters,med1way)
S3method(model_parameters,mediate)
S3method(model_parameters,merMod)
S3method(model_parameters,merModList)
S3method(model_parameters,meta_bma)
S3method(model_parameters,meta_fixed)
S3method(model_parameters,meta_random)
S3method(model_parameters,metaplus)
S3method(model_parameters,mhurdle)
S3method(model_parameters,mipo)
S3method(model_parameters,mira)
S3method(model_parameters,mixed)
S3method(model_parameters,mixor)
S3method(model_parameters,mjoint)
S3method(model_parameters,mle)
S3method(model_parameters,mle2)
S3method(model_parameters,mlm)
S3method(model_parameters,mmrm)
S3method(model_parameters,mmrm_fit)
S3method(model_parameters,mmrm_tmb)
S3method(model_parameters,model_fit)
S3method(model_parameters,multinom)
S3method(model_parameters,multinom_weightit)
S3method(model_parameters,mvord)
S3method(model_parameters,negbin)
S3method(model_parameters,negbinirr)
S3method(model_parameters,negbinmfx)
S3method(model_parameters,nestedLogit)
S3method(model_parameters,omega)
S3method(model_parameters,onesampb)
S3method(model_parameters,ordinal_weightit)
S3method(model_parameters,osrt)
S3method(model_parameters,pairwise.htest)
S3method(model_parameters,pam)
S3method(model_parameters,parameters_efa)
S3method(model_parameters,parameters_pca)
S3method(model_parameters,pb2)
S3method(model_parameters,pgmm)
S3method(model_parameters,poissonirr)
S3method(model_parameters,poissonmfx)
S3method(model_parameters,polr)
S3method(model_parameters,predictions)
S3method(model_parameters,principal)
S3method(model_parameters,probitmfx)
S3method(model_parameters,pvclust)
S3method(model_parameters,ridgelm)
S3method(model_parameters,rlmerMod)
S3method(model_parameters,rma)
S3method(model_parameters,robtab)
S3method(model_parameters,rqs)
S3method(model_parameters,rqss)
S3method(model_parameters,scam)
S3method(model_parameters,selection)
S3method(model_parameters,sem)
S3method(model_parameters,seqanova.svyglm)
S3method(model_parameters,slopes)
S3method(model_parameters,stanfit)
S3method(model_parameters,stanmvreg)
S3method(model_parameters,stanreg)
S3method(model_parameters,summary_emm)
S3method(model_parameters,survfit)
S3method(model_parameters,svy2lme)
S3method(model_parameters,svyglm)
S3method(model_parameters,svyolr)
S3method(model_parameters,svytable)
S3method(model_parameters,systemfit)
S3method(model_parameters,t1way)
S3method(model_parameters,trendPMCMR)
S3method(model_parameters,trimcibt)
S3method(model_parameters,varest)
S3method(model_parameters,vgam)
S3method(model_parameters,wbgee)
S3method(model_parameters,wbm)
S3method(model_parameters,wmcpAKP)
S3method(model_parameters,yuen)
S3method(model_parameters,zcpglm)
S3method(model_parameters,zerocount)
S3method(model_parameters,zeroinfl)
S3method(model_parameters,zoo)
S3method(p_calibrate,default)
S3method(p_calibrate,numeric)
S3method(p_direction,coxph)
S3method(p_direction,feis)
S3method(p_direction,felm)
S3method(p_direction,gee)
S3method(p_direction,glm)
S3method(p_direction,glmmTMB)
S3method(p_direction,gls)
S3method(p_direction,hurdle)
S3method(p_direction,lm)
S3method(p_direction,lme)
S3method(p_direction,merMod)
S3method(p_direction,mixed)
S3method(p_direction,rma)
S3method(p_direction,svyglm)
S3method(p_direction,wbm)
S3method(p_direction,zeroinfl)
S3method(p_significance,coxph)
S3method(p_significance,feis)
S3method(p_significance,felm)
S3method(p_significance,gee)
S3method(p_significance,glm)
S3method(p_significance,glmmTMB)
S3method(p_significance,gls)
S3method(p_significance,hurdle)
S3method(p_significance,lm)
S3method(p_significance,lme)
S3method(p_significance,merMod)
S3method(p_significance,mixed)
S3method(p_significance,parameters_model)
S3method(p_significance,rma)
S3method(p_significance,svyglm)
S3method(p_significance,wbm)
S3method(p_significance,zeroinfl)
S3method(p_value,BBmm)
S3method(p_value,BBreg)
S3method(p_value,BFBayesFactor)
S3method(p_value,BGGM)
S3method(p_value,DirichletRegModel)
S3method(p_value,Gam)
S3method(p_value,HLfit)
S3method(p_value,LORgee)
S3method(p_value,MCMCglmm)
S3method(p_value,Sarlm)
S3method(p_value,SemiParBIV)
S3method(p_value,aareg)
S3method(p_value,anova)
S3method(p_value,aov)
S3method(p_value,aovlist)
S3method(p_value,averaging)
S3method(p_value,bamlss)
S3method(p_value,bayesQR)
S3method(p_value,bayesx)
S3method(p_value,bcplm)
S3method(p_value,betamfx)
S3method(p_value,betaor)
S3method(p_value,betareg)
S3method(p_value,bife)
S3method(p_value,blavaan)
S3method(p_value,blrm)
S3method(p_value,bracl)
S3method(p_value,brmsfit)
S3method(p_value,brmultinom)
S3method(p_value,btergm)
S3method(p_value,censReg)
S3method(p_value,cgam)
S3method(p_value,clm2)
S3method(p_value,clmm2)
S3method(p_value,coeftest)
S3method(p_value,complmrob)
S3method(p_value,coxme)
S3method(p_value,coxph)
S3method(p_value,coxr)
S3method(p_value,cpglm)
S3method(p_value,cpglmm)
S3method(p_value,crch)
S3method(p_value,crq)
S3method(p_value,data.frame)
S3method(p_value,default)
S3method(p_value,deltaMethod)
S3method(p_value,draws)
S3method(p_value,eglm)
S3method(p_value,emmGrid)
S3method(p_value,emm_list)
S3method(p_value,externVar)
S3method(p_value,externX)
S3method(p_value,feglm)
S3method(p_value,fixest_multi)
S3method(p_value,flac)
S3method(p_value,flexsurvreg)
S3method(p_value,flic)
S3method(p_value,gam)
S3method(p_value,gamlss)
S3method(p_value,gamm)
S3method(p_value,gamm4)
S3method(p_value,gee)
S3method(p_value,geeglm)
S3method(p_value,glht)
S3method(p_value,glimML)
S3method(p_value,glmgee)
S3method(p_value,glmm)
S3method(p_value,glmx)
S3method(p_value,gls)
S3method(p_value,gmnl)
S3method(p_value,hglm)
S3method(p_value,htest)
S3method(p_value,hurdle)
S3method(p_value,ivFixed)
S3method(p_value,ivprobit)
S3method(p_value,ivreg)
S3method(p_value,lavaan)
S3method(p_value,lcmm)
S3method(p_value,list)
S3method(p_value,lm)
S3method(p_value,lm_robust)
S3method(p_value,lme)
S3method(p_value,lmodel2)
S3method(p_value,logistf)
S3method(p_value,logitmfx)
S3method(p_value,logitor)
S3method(p_value,lqm)
S3method(p_value,lqmm)
S3method(p_value,lrm)
S3method(p_value,margins)
S3method(p_value,maxLik)
S3method(p_value,mblogit)
S3method(p_value,mediate)
S3method(p_value,merMod)
S3method(p_value,metaplus)
S3method(p_value,mhurdle)
S3method(p_value,mipo)
S3method(p_value,mira)
S3method(p_value,mixor)
S3method(p_value,mjoint)
S3method(p_value,mle2)
S3method(p_value,mlm)
S3method(p_value,mmrm)
S3method(p_value,mmrm_fit)
S3method(p_value,mmrm_tmb)
S3method(p_value,model_fit)
S3method(p_value,multinom)
S3method(p_value,multinom_weightit)
S3method(p_value,mvord)
S3method(p_value,negbin)
S3method(p_value,negbinirr)
S3method(p_value,negbinmfx)
S3method(p_value,nestedLogit)
S3method(p_value,nlrq)
S3method(p_value,numeric)
S3method(p_value,ols)
S3method(p_value,ordinal_weightit)
S3method(p_value,pggls)
S3method(p_value,pglm)
S3method(p_value,plm)
S3method(p_value,poissonirr)
S3method(p_value,poissonmfx)
S3method(p_value,polr)
S3method(p_value,probitmfx)
S3method(p_value,psm)
S3method(p_value,riskRegression)
S3method(p_value,rlm)
S3method(p_value,rlmerMod)
S3method(p_value,rma)
S3method(p_value,rms)
S3method(p_value,robmixglm)
S3method(p_value,rq)
S3method(p_value,rqs)
S3method(p_value,rqss)
S3method(p_value,scam)
S3method(p_value,selection)
S3method(p_value,sem)
S3method(p_value,speedlm)
S3method(p_value,stanreg)
S3method(p_value,summary.lm)
S3method(p_value,survreg)
S3method(p_value,svy2lme)
S3method(p_value,svyglm)
S3method(p_value,svyglm.nb)
S3method(p_value,svyglm.zip)
S3method(p_value,svyolr)
S3method(p_value,systemfit)
S3method(p_value,tobit)
S3method(p_value,truncreg)
S3method(p_value,varest)
S3method(p_value,vgam)
S3method(p_value,vglm)
S3method(p_value,wbgee)
S3method(p_value,wbm)
S3method(p_value,zcpglm)
S3method(p_value,zerocount)
S3method(p_value,zeroinfl)
S3method(plot,cluster_analysis)
S3method(plot,cluster_analysis_summary)
S3method(plot,compare_parameters)
S3method(plot,equivalence_test_lm)
S3method(plot,n_clusters)
S3method(plot,n_clusters_dbscan)
S3method(plot,n_clusters_elbow)
S3method(plot,n_clusters_gap)
S3method(plot,n_clusters_hclust)
S3method(plot,n_clusters_silhouette)
S3method(plot,n_factors)
S3method(plot,parameters_brms_meta)
S3method(plot,parameters_efa)
S3method(plot,parameters_model)
S3method(plot,parameters_p_function)
S3method(plot,parameters_pca)
S3method(plot,parameters_sem)
S3method(plot,parameters_simulate)
S3method(predict,cluster_analysis)
S3method(predict,cluster_meta)
S3method(predict,kmeans)
S3method(predict,parameters_clusters)
S3method(predict,parameters_efa)
S3method(predict,parameters_pca)
S3method(predict,parameters_sem)
S3method(principal_components,data.frame)
S3method(principal_components,lm)
S3method(principal_components,merMod)
S3method(print,cfa_model)
S3method(print,cluster_analysis)
S3method(print,cluster_discrimination)
S3method(print,compare_parameters)
S3method(print,equivalence_test_lm)
S3method(print,n_clusters)
S3method(print,n_clusters_dbscan)
S3method(print,n_clusters_elbow)
S3method(print,n_clusters_gap)
S3method(print,n_clusters_hclust)
S3method(print,n_clusters_silhouette)
S3method(print,n_factors)
S3method(print,p_calibrate)
S3method(print,p_direction_lm)
S3method(print,p_significance_lm)
S3method(print,parameters_brms_meta)
S3method(print,parameters_clusters)
S3method(print,parameters_coef)
S3method(print,parameters_da)
S3method(print,parameters_efa)
S3method(print,parameters_efa_summary)
S3method(print,parameters_loadings)
S3method(print,parameters_model)
S3method(print,parameters_omega)
S3method(print,parameters_omega_summary)
S3method(print,parameters_p_function)
S3method(print,parameters_pca)
S3method(print,parameters_pca_summary)
S3method(print,parameters_random)
S3method(print,parameters_sem)
S3method(print,parameters_simulate)
S3method(print,parameters_standardized)
S3method(print_html,compare_parameters)
S3method(print_html,equivalence_test_lm)
S3method(print_html,parameters_brms_meta)
S3method(print_html,parameters_efa)
S3method(print_html,parameters_efa_summary)
S3method(print_html,parameters_model)
S3method(print_html,parameters_p_function)
S3method(print_html,parameters_pca)
S3method(print_html,parameters_pca_summary)
S3method(print_html,parameters_sem)
S3method(print_html,parameters_simulate)
S3method(print_html,parameters_standardized)
S3method(print_md,compare_parameters)
S3method(print_md,equivalence_test_lm)
S3method(print_md,parameters_brms_meta)
S3method(print_md,parameters_efa)
S3method(print_md,parameters_efa_summary)
S3method(print_md,parameters_model)
S3method(print_md,parameters_omega)
S3method(print_md,parameters_omega_summary)
S3method(print_md,parameters_p_function)
S3method(print_md,parameters_pca)
S3method(print_md,parameters_pca_summary)
S3method(print_md,parameters_sem)
S3method(print_md,parameters_simulate)
S3method(print_md,parameters_standardized)
S3method(reduce_parameters,data.frame)
S3method(reduce_parameters,lm)
S3method(reduce_parameters,merMod)
S3method(reshape_loadings,data.frame)
S3method(reshape_loadings,parameters_efa)
S3method(se_kenward,default)
S3method(se_kenward,glmmTMB)
S3method(se_satterthwaite,default)
S3method(select_parameters,lm)
S3method(select_parameters,merMod)
S3method(simulate_model,LORgee)
S3method(simulate_model,MixMod)
S3method(simulate_model,betamfx)
S3method(simulate_model,betaor)
S3method(simulate_model,betareg)
S3method(simulate_model,biglm)
S3method(simulate_model,bracl)
S3method(simulate_model,brmultinom)
S3method(simulate_model,censReg)
S3method(simulate_model,cglm)
S3method(simulate_model,clm)
S3method(simulate_model,clm2)
S3method(simulate_model,clmm2)
S3method(simulate_model,coxme)
S3method(simulate_model,coxph)
S3method(simulate_model,cpglm)
S3method(simulate_model,cpglmm)
S3method(simulate_model,crch)
S3method(simulate_model,crq)
S3method(simulate_model,default)
S3method(simulate_model,feglm)
S3method(simulate_model,feis)
S3method(simulate_model,fixest)
S3method(simulate_model,fixest_multi)
S3method(simulate_model,flac)
S3method(simulate_model,flexsurvreg)
S3method(simulate_model,flic)
S3method(simulate_model,gam)
S3method(simulate_model,gamlss)
S3method(simulate_model,gamm)
S3method(simulate_model,gee)
S3method(simulate_model,geeglm)
S3method(simulate_model,glimML)
S3method(simulate_model,glm)
S3method(simulate_model,glmRob)
S3method(simulate_model,glmmTMB)
S3method(simulate_model,glmmadmb)
S3method(simulate_model,glmrob)
S3method(simulate_model,glmx)
S3method(simulate_model,gls)
S3method(simulate_model,hurdle)
S3method(simulate_model,iv_robust)
S3method(simulate_model,ivreg)
S3method(simulate_model,list)
S3method(simulate_model,lm)
S3method(simulate_model,lmRob)
S3method(simulate_model,lm_robust)
S3method(simulate_model,lme)
S3method(simulate_model,lmrob)
S3method(simulate_model,logistf)
S3method(simulate_model,lrm)
S3method(simulate_model,merMod)
S3method(simulate_model,mhurdle)
S3method(simulate_model,mixor)
S3method(simulate_model,mlm)
S3method(simulate_model,model_fit)
S3method(simulate_model,multinom)
S3method(simulate_model,mvord)
S3method(simulate_model,nestedLogit)
S3method(simulate_model,nlrq)
S3method(simulate_model,ols)
S3method(simulate_model,plm)
S3method(simulate_model,polr)
S3method(simulate_model,psm)
S3method(simulate_model,rms)
S3method(simulate_model,rq)
S3method(simulate_model,selection)
S3method(simulate_model,speedglm)
S3method(simulate_model,speedlm)
S3method(simulate_model,survreg)
S3method(simulate_model,svyglm.nb)
S3method(simulate_model,svyglm.zip)
S3method(simulate_model,tobit)
S3method(simulate_model,truncreg)
S3method(simulate_model,varest)
S3method(simulate_model,vgam)
S3method(simulate_model,vglm)
S3method(simulate_model,zerocount)
S3method(simulate_model,zeroinfl)
S3method(simulate_parameters,default)
S3method(simulate_parameters,glmmTMB)
S3method(simulate_parameters,hurdle)
S3method(simulate_parameters,mblogit)
S3method(simulate_parameters,mlm)
S3method(simulate_parameters,multinom)
S3method(simulate_parameters,nestedLogit)
S3method(simulate_parameters,varest)
S3method(simulate_parameters,zerocount)
S3method(simulate_parameters,zeroinfl)
S3method(sort,parameters_efa)
S3method(sort,parameters_pca)
S3method(sort_parameters,data.frame)
S3method(sort_parameters,default)
S3method(standard_error,BBmm)
S3method(standard_error,BBreg)
S3method(standard_error,DirichletRegModel)
S3method(standard_error,HLfit)
S3method(standard_error,LORgee)
S3method(standard_error,MCMCglmm)
S3method(standard_error,MixMod)
S3method(standard_error,Sarlm)
S3method(standard_error,SemiParBIV)
S3method(standard_error,aareg)
S3method(standard_error,anova)
S3method(standard_error,aov)
S3method(standard_error,aovlist)
S3method(standard_error,averaging)
S3method(standard_error,bamlss)
S3method(standard_error,bayesx)
S3method(standard_error,betamfx)
S3method(standard_error,betaor)
S3method(standard_error,betareg)
S3method(standard_error,bfsl)
S3method(standard_error,bife)
S3method(standard_error,biglm)
S3method(standard_error,blavaan)
S3method(standard_error,bracl)
S3method(standard_error,brmsfit)
S3method(standard_error,brmultinom)
S3method(standard_error,btergm)
S3method(standard_error,censReg)
S3method(standard_error,cgam)
S3method(standard_error,character)
S3method(standard_error,clm2)
S3method(standard_error,clmm2)
S3method(standard_error,coeftest)
S3method(standard_error,complmrob)
S3method(standard_error,coxme)
S3method(standard_error,coxph)
S3method(standard_error,coxr)
S3method(standard_error,cpglm)
S3method(standard_error,cpglmm)
S3method(standard_error,crch)
S3method(standard_error,crq)
S3method(standard_error,data.frame)
S3method(standard_error,default)
S3method(standard_error,deltaMethod)
S3method(standard_error,draws)
S3method(standard_error,effectsize_table)
S3method(standard_error,emmGrid)
S3method(standard_error,emm_list)
S3method(standard_error,estimate_contrasts)
S3method(standard_error,estimate_means)
S3method(standard_error,estimate_slopes)
S3method(standard_error,externVar)
S3method(standard_error,externX)
S3method(standard_error,factor)
S3method(standard_error,feglm)
S3method(standard_error,fitdistr)
S3method(standard_error,fixest)
S3method(standard_error,fixest_multi)
S3method(standard_error,flac)
S3method(standard_error,flexsurvreg)
S3method(standard_error,flic)
S3method(standard_error,gam)
S3method(standard_error,gamlss)
S3method(standard_error,gamm)
S3method(standard_error,gamm4)
S3method(standard_error,gee)
S3method(standard_error,geeglm)
S3method(standard_error,glht)
S3method(standard_error,glimML)
S3method(standard_error,glmgee)
S3method(standard_error,glmm)
S3method(standard_error,glmmTMB)
S3method(standard_error,glmx)
S3method(standard_error,gls)
S3method(standard_error,gmnl)
S3method(standard_error,hglm)
S3method(standard_error,htest)
S3method(standard_error,hurdle)
S3method(standard_error,ivFixed)
S3method(standard_error,ivprobit)
S3method(standard_error,ivreg)
S3method(standard_error,lavaan)
S3method(standard_error,lcmm)
S3method(standard_error,list)
S3method(standard_error,lm_robust)
S3method(standard_error,lme)
S3method(standard_error,lmodel2)
S3method(standard_error,logistf)
S3method(standard_error,logitmfx)
S3method(standard_error,logitor)
S3method(standard_error,lqm)
S3method(standard_error,lqmm)
S3method(standard_error,lrm)
S3method(standard_error,margins)
S3method(standard_error,maxLik)
S3method(standard_error,mblogit)
S3method(standard_error,mediate)
S3method(standard_error,merMod)
S3method(standard_error,merModList)
S3method(standard_error,meta_bma)
S3method(standard_error,meta_fixed)
S3method(standard_error,meta_random)
S3method(standard_error,metaplus)
S3method(standard_error,mhurdle)
S3method(standard_error,mipo)
S3method(standard_error,mira)
S3method(standard_error,mixor)
S3method(standard_error,mjoint)
S3method(standard_error,mle)
S3method(standard_error,mle2)
S3method(standard_error,mlm)
S3method(standard_error,mmrm)
S3method(standard_error,mmrm_fit)
S3method(standard_error,mmrm_tmb)
S3method(standard_error,model_fit)
S3method(standard_error,multinom)
S3method(standard_error,multinom_weightit)
S3method(standard_error,mvord)
S3method(standard_error,mvstanreg)
S3method(standard_error,negbin)
S3method(standard_error,negbinirr)
S3method(standard_error,negbinmfx)
S3method(standard_error,nestedLogit)
S3method(standard_error,nlrq)
S3method(standard_error,numeric)
S3method(standard_error,ols)
S3method(standard_error,ordinal_weightit)
S3method(standard_error,parameters_kurtosis)
S3method(standard_error,parameters_skewness)
S3method(standard_error,parameters_standardized)
S3method(standard_error,pgmm)
S3method(standard_error,plm)
S3method(standard_error,poissonirr)
S3method(standard_error,poissonmfx)
S3method(standard_error,polr)
S3method(standard_error,probitmfx)
S3method(standard_error,psm)
S3method(standard_error,riskRegression)
S3method(standard_error,rma)
S3method(standard_error,rms)
S3method(standard_error,robmixglm)
S3method(standard_error,rq)
S3method(standard_error,rqs)
S3method(standard_error,rqss)
S3method(standard_error,scam)
S3method(standard_error,selection)
S3method(standard_error,sem)
S3method(standard_error,stanreg)
S3method(standard_error,summary.lm)
S3method(standard_error,survreg)
S3method(standard_error,svy2lme)
S3method(standard_error,svyglm)
S3method(standard_error,svyglm.nb)
S3method(standard_error,svyglm.zip)
S3method(standard_error,svyolr)
S3method(standard_error,systemfit)
S3method(standard_error,table)
S3method(standard_error,tobit)
S3method(standard_error,truncreg)
S3method(standard_error,varest)
S3method(standard_error,vgam)
S3method(standard_error,vglm)
S3method(standard_error,wbgee)
S3method(standard_error,wbm)
S3method(standard_error,xtabs)
S3method(standard_error,zcpglm)
S3method(standard_error,zerocount)
S3method(standard_error,zeroinfl)
S3method(standardize_info,default)
S3method(standardize_parameters,bootstrap_model)
S3method(standardize_parameters,bootstrap_parameters)
S3method(standardize_parameters,default)
S3method(standardize_parameters,mediate)
S3method(standardize_parameters,model_fit)
S3method(standardize_parameters,parameters_model)
S3method(summary,cluster_analysis)
S3method(summary,n_clusters)
S3method(summary,n_factors)
S3method(summary,parameters_efa)
S3method(summary,parameters_model)
S3method(summary,parameters_omega)
S3method(summary,parameters_pca)
S3method(visualisation_recipe,cluster_analysis)
S3method(visualisation_recipe,cluster_analysis_summary)
S3method(visualisation_recipe,n_clusters_dbscan)
S3method(visualisation_recipe,n_clusters_elbow)
S3method(visualisation_recipe,n_clusters_gap)
S3method(visualisation_recipe,n_clusters_silhouette)
export(bootstrap_model)
export(bootstrap_parameters)
export(ci)
export(ci_betwithin)
export(ci_kenward)
export(ci_ml1)
export(ci_satterthwaite)
export(closest_component)
export(cluster_analysis)
export(cluster_centers)
export(cluster_discrimination)
export(cluster_meta)
export(cluster_performance)
export(compare_models)
export(compare_parameters)
export(confidence_curve)
export(consonance_function)
export(convert_efa_to_cfa)
export(degrees_of_freedom)
export(demean)
export(describe_distribution)
export(display)
export(dof)
export(dof_betwithin)
export(dof_kenward)
export(dof_ml1)
export(dof_satterthwaite)
export(dominance_analysis)
export(efa_to_cfa)
export(equivalence_test)
export(factor_analysis)
export(factor_scores)
export(format_df_adjust)
export(format_order)
export(format_p_adjust)
export(format_parameters)
export(get_scores)
export(kurtosis)
export(model_parameters)
export(n_clusters)
export(n_clusters_dbscan)
export(n_clusters_elbow)
export(n_clusters_gap)
export(n_clusters_hclust)
export(n_clusters_silhouette)
export(n_components)
export(n_factors)
export(n_parameters)
export(p_calibrate)
export(p_direction)
export(p_function)
export(p_significance)
export(p_value)
export(p_value_betwithin)
export(p_value_kenward)
export(p_value_ml1)
export(p_value_satterthwaite)
export(parameters)
export(parameters_type)
export(pool_parameters)
export(principal_components)
export(print_html)
export(print_md)
export(random_parameters)
export(reduce_data)
export(reduce_parameters)
export(rescale_weights)
export(reshape_loadings)
export(rotated_data)
export(se_kenward)
export(se_satterthwaite)
export(select_parameters)
export(simulate_model)
export(simulate_parameters)
export(skewness)
export(sort_parameters)
export(standard_error)
export(standardise_info)
export(standardise_parameters)
export(standardise_posteriors)
export(standardize_info)
export(standardize_names)
export(standardize_parameters)
export(standardize_posteriors)
export(supported_models)
export(visualisation_recipe)
importFrom(bayestestR,ci)
importFrom(bayestestR,equivalence_test)
importFrom(bayestestR,p_direction)
importFrom(bayestestR,p_significance)
importFrom(datawizard,demean)
importFrom(datawizard,describe_distribution)
importFrom(datawizard,kurtosis)
importFrom(datawizard,rescale_weights)
importFrom(datawizard,skewness)
importFrom(datawizard,visualisation_recipe)
importFrom(graphics,plot)
importFrom(insight,display)
importFrom(insight,n_parameters)
importFrom(insight,print_html)
importFrom(insight,print_md)
importFrom(insight,standardize_names)
importFrom(insight,supported_models)
