# model parameters -------------------

#' @export
model_parameters.ordinal_weightit <- model_parameters.clm2

#' @export
model_parameters.multinom_weightit <- model_parameters.bracl

# CI ---------------------

#' @export
ci.ordinal_weightit <- ci.clm2

#' @export
ci.multinom_weightit <- ci.bracl

# standard errors -----------------

#' @export
standard_error.ordinal_weightit <- standard_error.clm2

#' @export
standard_error.multinom_weightit <- standard_error.bracl

# p values ----------------

#' @export
p_value.ordinal_weightit <- p_value.clm2

#' @export
p_value.multinom_weightit <- p_value.bracl
