# ----------------------- insight -------------------------------------

#' @importFrom insight standardize_names
#' @export
insight::standardize_names

#' @importFrom insight supported_models
#' @export
insight::supported_models

#' @importFrom insight print_html
#' @export
insight::print_html

#' @importFrom insight print_md
#' @export
insight::print_md

#' @importFrom insight display
#' @export
insight::display

# ----------------------- datawizard -------------------------------------

#' @importFrom datawizard describe_distribution
#' @export
datawizard::describe_distribution

#' @importFrom datawizard demean
#' @export
datawizard::demean

#' @importFrom datawizard rescale_weights
#' @export
datawizard::rescale_weights

#' @importFrom datawizard visualisation_recipe
#' @export
datawizard::visualisation_recipe

#' @importFrom datawizard kurtosis
#' @export
datawizard::kurtosis

#' @importFrom datawizard skewness
#' @export
datawizard::skewness
