---
title: "Overview of Vignettes"
output:
  rmarkdown::html_vignette:
vignette: >
  %\VignetteIndexEntry{Overview of Vignettes}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r message=FALSE, warning=FALSE, include=FALSE}
library(knitr)
knitr::opts_chunk$set(
  echo = TRUE,
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  comment = "#>",
  eval = TRUE
)
```

All package vignettes are available at [https://easystats.github.io/parameters/](https://easystats.github.io/parameters/).

## Function Overview

* [Function Reference](https://easystats.github.io/parameters/reference/index.html)

## Description of Parameters

* [Summary of Model Parameters](https://easystats.github.io/parameters/articles/model_parameters.html)
* [Parameter and Model Standardization](https://easystats.github.io/parameters/articles/standardize_parameters_effsize.html)
* [Robust Estimation of Standard Errors, Confidence Intervals, and p-values](https://easystats.github.io/parameters/articles/model_parameters_robust.html)
* [Model Parameters for Multiply Imputed Repeated Analyses](https://easystats.github.io/parameters/articles/model_parameters_mice.html)
* [Analysing Longitudinal or Panel Data](https://easystats.github.io/parameters/articles/demean.html)

## Formatting, Printing and Plotting

* [Formatting Model Parameters](https://easystats.github.io/parameters/articles/model_parameters_formatting.html)
* [Printing Model Parameters](https://easystats.github.io/parameters/articles/model_parameters_print.html)
* [Plotting functions](https://easystats.github.io/see/articles/parameters.html)

## Dimension Reduction and Clustering

* [Feature Reduction (PCA, cMDS, ICA, ...)](https://easystats.github.io/parameters/articles/parameters_reduction.html)
* [Structural Models (EFA, CFA, SEM, ...)](https://easystats.github.io/parameters/articles/efa_cfa.html)
* [Selection of Model Parameters](https://easystats.github.io/parameters/articles/parameters_selection.html)
* [Clustering with easystats](https://easystats.github.io/parameters/articles/clustering.html)

## Plotting Functions

  * [Plotting functions in the **see** package](https://easystats.github.io/see/articles/parameters.html)
