# templates

    Code
      print(mp, pretty_names = "labels")
    Output
      # Fixed Effects
      
      Parameter                                 | Coefficient |   SE |         95% CI |     z |     p
      -----------------------------------------------------------------------------------------------
      (Intercept)                               |        8.72 | 3.45 | [ 1.95, 15.48] |  2.52 | 0.012
      elder's dependency [slightly dependent]   |       -1.00 | 3.61 | [-8.08,  6.08] | -0.28 | 0.782
      elder's dependency [moderately dependent] |        2.68 | 3.06 | [-3.32,  8.68] |  0.88 | 0.381
      elder's dependency [severely dependent]   |        3.88 | 3.00 | [-2.01,  9.77] |  1.29 | 0.197
      carer's level of education                |        1.14 | 0.90 | [-0.62,  2.90] |  1.27 | 0.204
    Message
      
      Uncertainty intervals (equal-tailed) and p-values (two-tailed) computed
        using a Wald z-distribution approximation.

