% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.data_frame}
\alias{.data_frame}
\title{help-functions}
\usage{
.data_frame(...)
}
\description{
help-functions
}
\keyword{internal}
