% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.compact_list}
\alias{.compact_list}
\title{remove NULL elements from lists}
\usage{
.compact_list(x)
}
\description{
remove NULL elements from lists
}
\keyword{internal}
