% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.compact_character}
\alias{.compact_character}
\title{remove empty string from character}
\usage{
.compact_character(x)
}
\description{
remove empty string from character
}
\keyword{internal}
