% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{fish}
\alias{fish}
\title{Sample data set}
\description{
A sample data set, used in tests and some examples.
}
\keyword{data}
