% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.find_most_common}
\alias{.find_most_common}
\title{Find most common occurence}
\usage{
.find_most_common(x)
}
\description{
Find most common occurence
}
\keyword{internal}
